/************************************************/
/*						*/
/*      Copyright:				*/
/*	 Jean-Marc Pigeon <jmp@safe.ca>	 2009	*/
/*						*/
/************************************************/
/* This program is free software; you can 	*/
/* redistribute it and/or modify it under the 	*/
/* terms of the GNU General Public License as	*/
/* published by the Free Software Foundation	*/
/* version 2 of the License			*/
/*						*/
/* This program is distributed in the hope that */
/* it will be useful, but WITHOUT ANY WARRANTY; */
/* without even the implied warranty of		*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR	*/
/* PURPOSE.  See the GNU General Public License	*/
/* for more details.				*/
/*						*/
/* You should have received a copy of the GNU	*/
/* General Public License along with this 	*/
/* program; if not, write to the Free Software	*/
/* Foundation, Inc., 51 Franklin Street,	*/
/* Fifth Floor, Boston, MA  02110-1301, USA.	*/
/************************************************/
/*						*/
/*	CHECKID:				*/
/*	Program purpose is to call PAM to check	*/
/*	about a user id (id/password).		*/
/*	This program est SUID.			*/
/*	If this program is called with '-p'	*/
/*	option, check about user password is	*/
/*	done via a POP connection.		*/
/*						*/
/************************************************/
#include	<stdio.h>
#include	<string.h>
#include	<malloc.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<security/pam_appl.h>
#include	"subrou.h"
#include	"gestls.h"

#define	APPNAME		"checkid"

#define	PASSWD		"passwd"

static char *locpass;	/*current checked passwd*/
static int pam_status;	/*PAM current status	*/

/*

*/
/************************************************/
/*						*/
/*	Program usage help			*/
/*						*/
/************************************************/
static void usage()

{
(void) fprintf(stderr,"%s Version %s\n",APPNAME,rou_getvers());
(void) fprintf(stderr,"usage: /bin/echo 'username 'passwd' |  %s [-p hostname[,port]]"
		      "[-d debug] "
		      "\n",APPNAME);
(void) fprintf(stderr,"\t\t-p hostname,port : POP-3 Protocol hostname and port \n");
(void) fprintf(stderr,"\t\t-d debug         : debug level\n");
(void) fprintf(stderr,"\t\t                 : to be used instead of PAM\n");
(void) fprintf(stderr,"username and password are expected\n");
(void) fprintf(stderr,"to be received via stdin device\n");
}
/*

*/
/************************************************/
/*						*/
/*	Routine called to get password as it	*/
/*	was type from keyboard.			*/
/*						*/
/************************************************/
static int pampass(int num_msg, const struct pam_message **msg,
                     struct pam_response **response, void *appdata_ptr)

{
struct pam_response *pr;
const struct pam_message *pm;

*response=(struct pam_response *)malloc(num_msg*sizeof(struct pam_response));
if (*response==(struct pam_response *)0)
  return(PAM_CONV_ERR);
(void) memset((void *)*response,0,num_msg*sizeof(struct pam_response));
for (pr=*response, pm = *msg; num_msg--; pr++, pm++) {
  switch (pm->msg_style) {
    case PAM_PROMPT_ECHO_ON:
    case PAM_PROMPT_ECHO_OFF:
      pr->resp=strdup(locpass);
      break;
    case PAM_TEXT_INFO:
    case PAM_ERROR_MSG:
      break;
    default:
      /* Something odd happened 			*/
      /* XXX - should free non-NULL response members	*/
      (void) free((void *)(*response));
      *response = NULL;
      return(PAM_CONV_ERR);
      break;
    }
  }
return(PAM_SUCCESS);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to prepare check a password	*/
/*	using a pop channel instead of PAM	*/
/*	PAM.					*/
/*						*/
/************************************************/
static int checkviapop(char *pophost,char *username,char *passwd)

{
#define	OPSR	"checkid.c:checkviapop,"
#define	MXWAIT	20	/*waiting for 20 second MAX	*/
int status;
int port;
char hostname[100];

status=false;
port=110;		/*POP-3 Port			*/
if (sscanf(pophost,"%95[^\n\r,],%d",hostname,&port)>=2) {
  TLSFILE *tls;

  if ((tls=tls_opentcp(hostname,port,MXWAIT))!=(TLSFILE *)0) {
    int phase;
    int proceed;

    phase=0;
    proceed=true;
    while (proceed==true) {
      char *buffer;

      if ((buffer=tls_waitforaction(MXWAIT,tls))==(char *)0) {
	break;			/*timeout			*/
	}
      switch (phase) {
	case 0		:	/*checking for signon		*/
	  if (*buffer=='+') {	/*and sending username		*/
	    (void) tls_printf(tls,"user %s\n",username);
	    }
	  else
            proceed=false;
	  break;
	case 1		:	/*checking for user accepted	*/
	  if (*buffer=='+') {	/*and sending password		*/
	    (void) tls_printf(tls,"pass %s\n",passwd);
	    break;
	    }
	  /*NO break if trouble					*/
	default		:	/*checking for password accepted*/
	  if (*buffer=='+')		
	    status=true;
				/*sending quit and exiting	*/
	  (void) tls_printf(tls,"quit\n");
	  (void) usleep(10000);
	  proceed=false;
	  break;
        }
      buffer=rou_freestr(buffer);
      phase++;
      }
    (void) tls_close(tls);
    }
  }
else {
  (void) alert(0,"%s Unable to scan <%s>",OPSR,pophost);
  }
return status;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to prepare PAMINFO and call	*/
/*	PAM.					*/
/*						*/
/************************************************/
static int checkpass(pam_handle_t *pamh,char *username,char *passwd)

{
int statut;
char *ptr;
struct pam_conv pam_conv;

statut=0;
if ((ptr=strchr(username,'@'))!=(char *)0)
  *ptr='\000';
pam_conv.conv=pampass;
(void) pam_set_item(pamh,PAM_USER,(const void *)username);
locpass=passwd;
switch (pam_status=pam_authenticate(pamh,PAM_SILENT)) {
  case PAM_SUCCESS	:
    statut=1;
    break;
  }
return statut;
}
/*
^L
*/
/************************************************/
/*						*/
/*	Program Entry.				*/
/*						*/
/************************************************/
int main(int argc,char *argv[])

{
int usepop;
struct pam_conv pam_conv;
char c;
pam_handle_t *pamh;
char pophost[100];
char ligne[300];

appname=APPNAME;
usepop=false;
pam_conv.conv=pampass;
while ((c=getopt(argc,argv,"d:p:"))!=EOF) {
  switch(c) {
    case   'd'  :
      debug=atoi(optarg);
      break;
    case   'p'  :
      usepop=true;
      (void) strncpy(pophost,optarg,sizeof(pophost));
      break;
    default	:
      (void) usage();
      break;
    }
  }
(void) fprintf(stderr,"uid='%d', euid='%d'\n",getuid(),geteuid());
if ((pam_status=pam_start(PASSWD,(void *)0,&pam_conv,&pamh))!=PAM_SUCCESS) {
  (void) fprintf(stderr,"Unable to open PAM Library for application '%s' (fatal)\n",appname);
  (void) exit(-1);
  }
while (fgets(ligne,sizeof(ligne),stdin)!=(char *)0) {
  char *username;
  char *password;
  
  username=(char *)calloc(strlen(ligne),sizeof(char));
  password=(char *)calloc(strlen(ligne),sizeof(char));
  if (sscanf(ligne,"%s %s",username,password)==2) {
    int statut;

    if (usepop==false) 
      statut=checkpass(pamh,username,password);
    else
      statut=checkviapop(pophost,username,password);
    (void) fprintf(stdout,"%d\n",statut);
    }
  (void) free(password);
  (void) free(username);
  }
(void) pam_end(pamh,pam_status);
return 0;
}
