/************************************************/
/*						*/
/*      Copyright:				*/
/*	 Jean-Marc Pigeon <jmp@safe.ca>	 2009	*/
/*						*/
/************************************************/
/* This program is free software; you can 	*/
/* redistribute it and/or modify it under the 	*/
/* terms of the GNU General Public License as	*/
/* published by the Free Software Foundation	*/
/* version 2 of the License			*/
/*						*/
/* This program is distributed in the hope that */
/* it will be useful, but WITHOUT ANY WARRANTY; */
/* without even the implied warranty of		*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR	*/
/* PURPOSE.  See the GNU General Public License	*/
/* for more details.				*/
/*						*/
/* You should have received a copy of the GNU	*/
/* General Public License along with this 	*/
/* program; if not, write to the Free Software	*/
/* Foundation, Inc., 51 Franklin Street,	*/
/* Fifth Floor, Boston, MA  02110-1301, USA.	*/
/************************************************/
/*						*/
/*	FEEDER:					*/
/*	Program to feed SMTP channel with file	*/
/*	contents.				*/
/*	This program is for test purpose, to	*/
/*	check if an E-mail contents is properly	*/
/*	mamanged by clement.			*/
/*						*/
/************************************************/
#include	<stdlib.h>
#include	<unistd.h>
#include	<string.h>
#include	<getopt.h>
#include	<syslog.h>
#include	"subrou.h"
#include	"unidev.h"
#include	"unieml.h"
#include	"gestls.h"
#include	"gesque.h"
#include	"lvleml.h"


#define	FEEDER	"feeder"
#define	SMTP	25

typedef	enum	{	/*SMTP Keyword code	*/
	c_ehlo,		/*EHLO			*/
	c_helo,		/*HELO			*/
	c_mail,		/*MAIL FROM:		*/
	c_rcpt,		/*RCPT TO:		*/
	c_reset,	/*RSET			*/
	c_data,		/*DATA			*/
	c_quit,		/*QUIT			*/
	c_orgn,		/*ORGN (clement added)	*/ 
	c_unknown	/*Unkonwn SMTP code	*/
	}CODENUM;
/*

*/
/************************************************/
/*						*/
/*	Procedure to display the program usage	*/
/*						*/
/************************************************/
static void usage()

{
(void) fprintf(stderr,"%s Version %s\n",appname,rou_getvers());
(void) fprintf(stderr,"usage:  %s "
		      "[-b base] "
		      "[-d debug] "
		      "[-h ] "
		      "[-p ] "
		      "[-v ] "
		      "server [filenames...]\n",appname);
(void) fprintf(stderr,"\t\t-b base      : application working directory\n");
(void) fprintf(stderr,"\t\t-d debug     : debug level\n");
(void) fprintf(stderr,"\t\t-h           : display usage\n");
(void) fprintf(stderr,"\t\t-s port      : waiting SMTP port\n");
(void) fprintf(stderr,"\t\t-v           : verbose debug\n");
(void) fprintf(stderr,"\t\t             : if [filenames] is missing, filename are\n");
(void) fprintf(stderr,"\t\t             : taken from standard input\n");
(void) exit(-1);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to get the CODENUM related	*/
/*	tp SMTP syntaxe.			*/
/*						*/
/************************************************/
static CODENUM getcode(char *line)

{
static	struct	{
	CODENUM	code;
	char *key;
	}voca[]={
	{c_ehlo,"ehlo"},
	{c_helo,"helo"},
	{c_mail,"mail from:"},
	{c_rcpt,"rcpt to:"},
	{c_reset,"reset"},
	{c_data,"data"},
	{c_quit,"quit"},
	{c_orgn,"orgn"},
	{c_unknown,(char *)0}
	};

CODENUM code;
int i;

code=c_unknown;
(void) rou_strtolower(line);
for (i=0;voca[i].key!=(char *)0;i++) {
  if (strstr(line,voca[i].key)==line) {
    code=voca[i].code;
    break;
    }
  }
return code;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to check an answer from the	*/
/*	remote SMTP server.			*/
/*						*/
/************************************************/
static int getanswer(TLSFILE *tls,int expected)

{
#define WAITING	300

char *line;
int status;
int gotit;

status=false;
gotit=false;
while (gotit==false) {
  if ((line=tls_waitforaction(WAITING,tls))!=(char *)0) {
    int code;
    char sep;

    code=-1;
    (void) que_cleanstring(line);
    (void) tracing(TRALOG,5,"feeder got <%s>",line);
    if (sscanf(line,"%d%c",&code,&sep)==2) {
      if (sep=='-')
	continue;
      gotit=true;
      status=true;
      if (expected!=code) {
        status=false;
        (void) tracing(TRALOG,0,"Expected '%d' received '%s'",expected,line);
        }
      }
    line=rou_freestr(line);
    }
  }
return status;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to send SMTP sequence to a	*/
/*	mentor daemon.				*/
/*						*/
/************************************************/
static void envoit(CTTYP *contact,char *filename)

{
FILE *fichier;
char fullname[300];

(void) fprintf(stdout,"Sending file <%s>\n",filename);
(void) sprintf(fullname,"%s/%s",base,filename);
if ((fichier=fopen(fullname,"r"))==(FILE *)0) {
  (void) tracing(TRALOG,0,"%s, Unable to open file '%s'",appname,fullname);
  }
else {
  int numrcpt;
  int datamode;
  int proceed;

  numrcpt=0;
  datamode=false;
  proceed=true;
  while (proceed==true) {
    int expected;

    expected=QUITOK;
    if (datamode==false) {
      CODENUM code;
      char line[1600];

      expected=CMDOK;
      if (rou_getstr(fichier,line,sizeof(line),'\000')==(char *)0) {
	proceed=false;
	break;
	}
      switch((code=getcode(line))) {
	case c_ehlo	:
	case c_helo	:
	case c_orgn	:
	case c_mail	:
	case c_reset	:
	  break;
	case c_rcpt	:
	  break;
	case c_quit	:
	  expected=QUITOK;
	  break;
	case c_data	:
	  expected=DATAOK;
	  datamode=true;
	  break;
	default		:
	  (void) tracing(TRALOG,0,"sending, Unexpected eml_getcode "
				  "value ('%s')",line);
          proceed=false;
	  break;
	}
      if (proceed==true) {
        (void) eml_printfcrlf(contact->tls,"%s",line);
        proceed=getanswer(contact->tls,expected);
        }
      switch (code) {
	case c_mail	:
	  break;
	case c_rcpt	:
	  if (proceed==true)
	    numrcpt++;
	  proceed=true;
	default		:
	  break;
        }
      }
    else {
      char line[1600];
 
      expected=CMDOK; 
      while (rou_getstr(fichier,line,sizeof(line),'\000')!=(char *)0) {
        (void) tracing(TRALOG,7,"sending <%s>",line);
	(void) eml_printfcrlf(contact->tls,"%s",line);
	if ((line[0]=='.')&&(line[1]=='\000')) {
          proceed=getanswer(contact->tls,expected);
	  break;
	  }
	}
      datamode=false;
      }
    if (proceed==false) {
      (void) eml_printfcrlf(contact->tls,"%s","QUIT");
      (void) usleep(3000);
      }
    if (expected==QUITOK)
      proceed=false;
    }
  (void) fclose(fichier);
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to wakeup SMTP daemon		*/
/*	SMTP file to it.			*/
/*						*/
/************************************************/
static CTTYP *towakeup(char *host,int port)

{
#define	MXWAIT	60

int handle;
CTTYP *contact;

handle=-1;
if ((contact=eml_openmails(host,port,MXWAIT))==(CTTYP *)0) {
  (void) alert(0,"%s, Unable to contact host '%s' on port '%d'",
		 appname,host,port);
  }
return contact;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to wakeup SMTP daemon and send*/
/*	SMTP file to it.			*/
/*						*/
/************************************************/
static void sending(char *host,int port,char *filename)

{
CTTYP *contact;

if ((contact=towakeup(host,port))==(TLSFILE *)0) {
  return;
  } 
(void) envoit(contact,filename);
contact=eml_itsover(contact);
}
/*

*/
/************************************************/
/*						*/
/*	Main Program				*/
/*						*/
/************************************************/
int main(int argc,char *argv[])

{
char c;
int chkleak;
int port;
char *host;
char curpath[200];

chkleak=false;
port=SMTP;
host=(char *)0;
appname=FEEDER;
base=strdup(base);
(void) strncpy(curpath,argv[0],sizeof(curpath));
(void) openlog(appname,LOG_PID,0);
while ((c=getopt(argc,argv,"b:d:h:s:v"))!=EOF) {
  switch(c) {
    case   'b'  :
      (void) free(base);
      base=strdup(optarg);
      break;
    case   'd'  :
      debug=atoi(optarg);
      break;
    case   's'  :
      port=atoi(optarg);
      break;
    case   'v'  :
      verbose++;
      break;
    case   'h'  :
    default     :
      (void) usage();
      break;
    }
  }
if (optind+1>argc) {
  (void) usage();
  }
host=argv[optind];
optind++;
if (argc>optind) {
  int i;
  
  for (i=optind;i<argc;i++) {
    (void) sending(host,port,argv[i]);
    }
  }
else {
  CTTYP *contact;

  if ((contact=towakeup(host,port))!=(TLSFILE *)0) {
    char str[16000];

    while (fgets(str,sizeof(str),stdin)!=(char *)0) {
      while (str[strlen(str)-1]=='\n') 
        str[strlen(str)-1]='\000';
      (void) envoit(contact->tls,str);
      }
    contact=eml_itsover(contact);
    }
  }
return 0;
}
